/*
            +------------------------------+
$0000_0000: |                              |
            |          32k Hub RAM         |
$0000_7FFF: |                              |
            +------------------------------+
$0000_8000: |                              |
            |      16k Character ROM       |
$0000_BFFF: |                              |
            +------------------------------+
$0000_C000: |                              |
            |   16k Trig/Spin/Booter ROM   |
            |        unscrambled.hex       |
$0000_FFFF: |                              |
            +------------------------------+
*/
// hub_mem

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

module              hub_mem
(
input               clk_cog,
input               ena_bus,

input               w,
input        [3:0]  wb,
input       [13:0]  a,
input       [31:0]  d,

output      [31:0]  q
);


// 8192 x 32 ram with byte-write enables ($0000..$7FFF)  // <=- hub ram size (32k)

reg [7:0] ram3 [8191:0];  // <=- hub ram size (32k)
reg [7:0] ram2 [8191:0];  // <=- hub ram size (32k)
reg [7:0] ram1 [8191:0];  // <=- hub ram size (32k)
reg [7:0] ram0 [8191:0];  // <=- hub ram size (32k)


reg [7:0] ram_q3;
reg [7:0] ram_q2;
reg [7:0] ram_q1;
reg [7:0] ram_q0;

wire romh = &a[13:12];  // <=- rom/ram decode

wire roml =  a[13] && !a[12];  // <=- rom/ram decode

wire rom = romh || roml;  // <=- rom/ram decode

always @(posedge clk_cog)
begin
    if (ena_bus && !rom && w && wb[3])  // <=- rom/ram decode
        ram3[a[12:0]] <= d[31:24];  // <=- rom/ram decode
    if (ena_bus && !rom)  // <=- rom/ram decode
        ram_q3 <= ram3[a[12:0]];  // <=- rom/ram decode
end

always @(posedge clk_cog)
begin
    if (ena_bus && !rom && w && wb[2])  // <=- rom/ram decode
        ram2[a[12:0]] <= d[23:16];  // <=- rom/ram decode
    if (ena_bus && !rom)  // <=- rom/ram decode
        ram_q2 <= ram2[a[12:0]];  // <=- rom/ram decode
end

always @(posedge clk_cog)
begin
    if (ena_bus && !rom && w && wb[1])  // <=- rom/ram decode
        ram1[a[12:0]] <= d[15:8];  // <=- rom/ram decode
    if (ena_bus && !rom)  // <=- rom/ram decode
        ram_q1 <= ram1[a[12:0]];  // <=- rom/ram decode
end

always @(posedge clk_cog)
begin
    if (ena_bus && !rom && w && wb[0])  // <=- rom/ram decode
        ram0[a[12:0]] <= d[7:0];  // <=- rom/ram decode
    if (ena_bus && !rom)  // <=- rom/ram decode
        ram_q0 <= ram0[a[12:0]];  // <=- rom/ram decode
end


// 4096 x 32 rom containing character definitions ($8000..$BFFF)

(* ram_init_file = "hub_rom_low.hex" *)     reg [31:0] rom_low [4095:0];

reg [31:0] rom_low_q;

always @(posedge clk_cog)
if (ena_bus && a[13:12] == 2'b10)
    rom_low_q <= rom_low[a[11:0]];

// 4096 x 32 rom containing sin table, log table, booter, and interpreter ($C000..$FFFF)  <=-  high rom size

(* ram_init_file = "unscrambled.hex" *)    reg [31:0] rom_high [4095:0];  // <=- high rom size & file

reg [31:0] rom_high_q;

always @(posedge clk_cog)
if (ena_bus && a[13:12] == 2'b11)
    rom_high_q <= rom_high[a[11:0]];



// memory output mux

reg romhc;  // <=- decode
reg romlc;  // <=- decode

always @(posedge clk_cog)
if (ena_bus)
        romhc <= romh;    // <=- decode


always @(posedge clk_cog)
if (ena_bus)
        romlc <= roml;    // <=- decode

assign q = romhc ? rom_high_q
       : romlc ? rom_low_q
       : {ram_q3, ram_q2, ram_q1, ram_q0};

endmodule

